"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const stream_1 = require("stream");
const path_1 = require("path");
const fileSystemHandler_1 = require("../utils/fileSystemHandler");
const fs_1 = require("fs");
const util_1 = require("util");
const errors_1 = require("../errors");
const transformers_1 = require("./transformers");
const archiver_1 = require("archiver");
exports.pipeline = util_1.promisify(stream_1.pipeline);
class ComponentReader extends stream_1.Readable {
    constructor(components) {
        super({ objectMode: true });
        this.currentIndex = 0;
        this.components = components;
    }
    _read() {
        if (this.currentIndex < this.components.length) {
            const c = this.components[this.currentIndex];
            this.currentIndex += 1;
            this.push(c);
        }
        else {
            this.push(null);
        }
    }
}
exports.ComponentReader = ComponentReader;
class ComponentConverter extends stream_1.Transform {
    constructor(targetFormat) {
        super({ objectMode: true });
        this.targetFormat = targetFormat;
    }
    _transform(chunk, encoding, callback) {
        let err;
        let result;
        try {
            const transformer = transformers_1.getTransformer(chunk);
            switch (this.targetFormat) {
                case 'metadata':
                    result = transformer.toMetadataFormat();
                    break;
                case 'source':
                    result = transformer.toSourceFormat();
                    break;
                default:
                    throw new errors_1.LibraryError('error_convert_invalid_format', this.targetFormat);
            }
        }
        catch (e) {
            err = e;
        }
        callback(err, result);
    }
}
exports.ComponentConverter = ComponentConverter;
class ComponentWriter extends stream_1.Writable {
    constructor(rootDestination) {
        super({ objectMode: true });
        this.rootDestination = rootDestination;
    }
}
exports.ComponentWriter = ComponentWriter;
class StandardWriter extends ComponentWriter {
    constructor(rootDestination) {
        super(rootDestination);
    }
    _write(chunk, encoding, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            try {
                const writeTasks = chunk.writeInfos.map((info) => {
                    const fullDest = path_1.join(this.rootDestination, info.relativeDestination);
                    fileSystemHandler_1.ensureFileExists(fullDest);
                    return exports.pipeline(info.source, fs_1.createWriteStream(fullDest));
                });
                // it is a reasonable expectation that when a conversion call exits, the files of
                // every component has been written to the destination. This await ensures the microtask
                // queue is empty when that call exits and overall less memory is consumed.
                yield Promise.all(writeTasks);
            }
            catch (e) {
                err = e;
            }
            callback(err);
        });
    }
}
exports.StandardWriter = StandardWriter;
class ZipWriter extends ComponentWriter {
    constructor(rootDestination) {
        super(rootDestination);
        // compression-/speed+ (0)<---(3)---------->(9) compression+/speed-
        // 3 appears to be a decent balance of compression and speed. It felt like
        // higher values = diminishing returns on compression and made conversion slower
        this.zip = archiver_1.create('zip', { zlib: { level: 3 } });
        this.buffers = [];
        exports.pipeline(this.zip, this.getOutputStream());
    }
    _write(chunk, encoding, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            try {
                for (const writeInfo of chunk.writeInfos) {
                    this.addToZip(writeInfo.source, writeInfo.relativeDestination);
                }
            }
            catch (e) {
                err = e;
            }
            callback(err);
        });
    }
    _final(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let err;
            try {
                yield this.zip.finalize();
            }
            catch (e) {
                err = e;
            }
            callback(err);
        });
    }
    addToZip(contents, path) {
        this.zip.append(contents, { name: path });
    }
    getOutputStream() {
        if (this.rootDestination) {
            return fs_1.createWriteStream(this.rootDestination);
        }
        else {
            const bufferWritable = new stream_1.Writable();
            bufferWritable._write = (chunk, encoding, cb) => {
                this.buffers.push(chunk);
                cb();
            };
            return bufferWritable;
        }
    }
    get buffer() {
        return Buffer.concat(this.buffers);
    }
}
exports.ZipWriter = ZipWriter;
//# sourceMappingURL=streams.js.map